-- **************************************************************************
--   TitanPalsConfig.lua
--
--   By: KanadiaN
--        (Lindarena@Laughing Skull)
-- **************************************************************************

local L = LibStub( "AceLocale-3.0" ):GetLocale( "Titan", true );
local LB = LibStub( "AceLocale-3.0" ):GetLocale( "TitanPals", true );
local AceConfigDialog = LibStub( "AceConfigDialog-3.0" );
local AceConfig = LibStub( "AceConfig-3.0" );

-- **************************************************************************
-- NAME : Core Vars
-- DESC : These are here so they do not get changed like the locales would
-- **************************************************************************

palsDB = nil;
palsAuthor = nil;
TitanPalsFormatToRemove = "None";

local PlugInName = "TitanPals";
LB["TITAN_PALS_CORE_ID"] = "Pals";
LB["TITAN_PALS_CORE_VERSION"] = GetAddOnMetadata( PlugInName, "Version" );
LB["TITAN_PALS_CORE_DTAG"] = "|cffff0000<TitanPals Debug>|r %s";
LB["TITAN_PALS_CORE_ITAG"] = "|cffff9900<TitanPals>|r %s";
LB["TITAN_PALS_CORE_ETAG"] = "|cffff9900<TitanPals Error>|r %s";
LB["TITAN_PALS_CORE_ADDVER"] = "r74";
LB["TITAN_PALS_CORE_DBVER"] = "v2";
LB["TITAN_PALS_CORE_ARTWORK"] = "Interface\\FriendsFrame\\UI-Toast-ChatInviteIcon";
LB["TITAN_PALS_CORE_DEBUG_HEADER"] = "Debugging"
LB["TITAN_PALS_CORE_DEBUG_EVENT"] = "Events"
LB["TITAN_PALS_CORE_DEBUG_EVENT_DESC"] = "Displays event as they happen in the chat frame"
LB["TITAN_PALS_CORE_DEBUG_LOGEVENT"] = "Log Event"
LB["TITAN_PALS_CORE_DEBUG_LOGEVENT_DESC"] = "This will log the events as they happen for later veiwing"
LB["TITAN_PALS_CORE_DEBUG_HEADER_PRINT"] = "Print Functions"
LB["TITAN_PALS_CORE_DEBUG_PRINT_ALTS"] = "Print Alts"
LB["TITAN_PALS_CORE_DEBUG_PRINT_ALTS_DESC"] = "This will print a list of your alts to the chat frame"
LB["TITAN_PALS_CORE_DEBUG_PRINT_FRIENDS"] = "Print Friends"
LB["TITAN_PALS_CORE_DEBUG_PRINT_FRIENDS_DESC"] = "This will print a list of your friends to the chat frame"
LB["TITAN_PALS_CORE_DEBUG"] = "Debug"
LB["TITAN_PALS_CORE_DEBUG_DESC"] = "Enable Debug Mode"
LB["TITAN_PALS_CORE_DEBUG_REMAUTH"] = "Remove Author"
LB["TITAN_PALS_CORE_DEBUG_REMAUTH_DESC"] = "Removes |cffff0000%s|r as a Addon Author"
LB["TITAN_PALS_CORE_ADDAUTHOR"] = "|cff00ff00%s|r Has been added as a Author"
LB["TITAN_PALS_CORE_REMAUTHOR"] = "|cffff0000%s|r Has been removed as a Author"

LB["TITAN_PALS_HELP_ADDAUTH"] = "  |cffff0000/%s AddAuth|r - Add a AddOn Author to Data Array"
LB["TITAN_PALS_HELP_REMAUTH"] = "  |cffff0000/%s RemAuth|r - Remove a AddOn Author from the Data Array"
LB["TITAN_PALS_CORE_MINLEVEL"] = 0
LB["TITAN_PALS_CORE_MAXLEVEL"] = 85



-- **************************************************************************
-- NAME : TitanPanelPals_ConfigUtils(a)
-- DESC : Pull information from .toc for config dialog
-- **************************************************************************
local function TitanPanelPals_ConfigUtils( a )
     if ( a == "GetAuthor" ) then
          return GetAddOnMetadata( PlugInName, "Authors" );
     elseif ( a == "GetTranslation" ) then
          return GetAddOnMetadata( PlugInName, "X-Translation" );
     elseif ( a == "GetCategory" ) then
          return GetAddOnMetadata( PlugInName, "X-Category" );
     elseif ( a == "GetEmail" ) then
          return GetAddOnMetadata( PlugInName, "X-Email" );
     elseif ( a == "GetWebsite" ) then
          return GetAddOnMetadata( PlugInName, "X-Website" );
     elseif ( a == "GetVersion" ) then
          return tostring( GetAddOnMetadata( PlugInName, "Version" ) );
     end
end

-- **************************************************************************
-- NAME : PalsOpt
-- DESC : This is the root of the config dialog
-- **************************************************************************
local PalsOpt = {
     type = "group",
     name = LB["TITAN_PALS_ADDON_LABEL"],
     args = {
          confgendesc = {
               type = "description", order = 1,
               name = LB["TITAN_PALS_ABOUT_DESC"],
               cmdHidden = true
          },
	  cat1 = {

               type = "group", order = 2, inline = true,
               name = LB["TITAN_PALS_ABOUT"],
               args = {
                    confversiondesc = {
                         type = "description", order = 1,
                         name = LB["TITAN_PALS_ABOUT_VERSION"]:format( TitanPanelPals_ConfigUtils( "GetVersion" ) ),
                         cmdHidden = true
                    },
                    confauthordesc = {
                         type = "description", order = 2,
                         name = LB["TITAN_PALS_ABOUT_AUTHOR"]:format( TitanPanelPals_ConfigUtils( "GetAuthor" ) ),
                         cmdHidden = true
                    },
                    conflangauthordesc = {
                         type = "description", order = 3,
                         name = LB["TITAN_PALS_ABOUT_TRANSLATION"]:format( TitanPanelPals_ConfigUtils( "GetTranslation" ) ),
                         cmdHidden = true
                    },
                    confcatdesc = {
                         type = "description", order = 4,
                         name = LB["TITAN_PALS_ABOUT_CATEGORY"]:format( TitanPanelPals_ConfigUtils( "GetCategory" ) ),
                         cmdHidden = true
                    },
                    confemaildesc = {
                         type = "description", order = 5,
                         name = LB["TITAN_PALS_ABOUT_EMAIL"]:format( TitanPanelPals_ConfigUtils( "GetEmail" ) ),
                         cmdHidden = true
                    },
                    confwebsitedesc = {
                         type = "description", order = 6,
                         name = LB["TITAN_PALS_ABOUT_WEBSITE"]:format( TitanPanelPals_ConfigUtils( "GetWebsite" ) ),
                         cmdHidden = true
                    },
                    confmemorydesc = {
                         type = "description", order = 7,
                         name = function()
                              UpdateAddOnMemoryUsage();
                              mem = floor( GetAddOnMemoryUsage( PlugInName ) );
                              return LB["TITAN_PALS_ABOUT_MEMORY"]:format( mem );
                         end,
                         cmdHidden = true,
                    },
               },
	  },
	  cat2 = {
	       type = "group", order = 3,
	       name = LB["TITAN_PALS_CONFIG_HEADER_SETTINGS"],
	       args = {
                    tooltipshowignored = {
                         type = "toggle", order = 1,
                         name = LB["TITAN_PALS_CONFIG_SHOWIGNORED"],
                         desc = LB["TITAN_PALS_CONFIG_SHOWIGNORED_DESC"],
                         get = function() return TitanPals.Settings.ShowIgnored; end,
                         set = function( _, value )
                              TitanPals.Settings.ShowIgnored = value;
                              TitanPanelButton_UpdateButton( LB["TITAN_PALS_CORE_ID"] );
                         end,
                    },
                    tooltipshowmem = {
                         type = "toggle", order = 2,
                         name = LB["TITAN_PALS_CONFIG_SHOWMEM"],
                         desc = LB["TITAN_PALS_CONFIG_SHOWMEM_DESC"],
                         get = function() return TitanPals.Settings.ShowMem; end,
                         set = function( _, value ) TitanPals.Settings.ShowMem = value; end,
                    },
                    tooltipbannor = {
                         type = "toggle", order = 2,
                         name = LB["TITAN_PALS_CONFIG_BANNOR"],
                         desc = LB["TITAN_PALS_CONFIG_BANNOR_DESC"],
                         get = function() return TitanPals.Settings.DisplayB; end,
                         set = function( _, value ) TitanPals.Settings.DisplayB = value; end,
                    },
		    tooltipshowoffline = {
                         type = "toggle", order = 4,
                         name = LB["TITAN_PALS_CONFIG_SHOWOFFLINE"],
                         desc = LB["TITAN_PALS_CONFIG_SHOWOFFLINE_DESC"],
                         get = function() return TitanPals.Settings.ShowOffline; end,
                         set = function( _, value ) TitanPals.Settings.ShowOffline = value; end,
                    },
                    tooltipnoalts = {
                         type = "toggle", order = 5, width = "full",
                         name = LB["TITAN_PALS_CONFIG_NOALTS"],
                         desc = LB["TITAN_PALS_CONFIG_NOALTS_DESC"],
                         get = function() return TitanPals.Settings.NoAlts; end,
                         set = function( _, value ) TitanPals.Settings.NoAlts = value; end,
                    },
                    debugmode = {
                         type = "toggle", order = 6,
                         name = LB["TITAN_PALS_CORE_DEBUG"],
                         desc = LB["TITAN_PALS_CORE_DEBUG_DESC"],
                         hidden = function() return not TitanPals_CoreUtils( "Author", ( "%s@%s" ):format( TitanPals.Profile.Player, TitanPals.Profile.Realm ) ) end,
                         get = function() return TitanPals.Debug.State; end,
                         set = function( _, value ) TitanPals.Debug.State = value; end,
                    },
               },
	  },
	  cat3 = {
               type = "group", order = 4,
               name = LB["TITAN_PALS_CONFIG_HEADER_TOOLTIP"],
               args = {
	            tooltippresets = {
                         type = "select", order = 1, width = "full",
                         name = LB["TITAN_PALS_CONFIG_PRESETS"],
                         desc = LB["TITAN_PALS_CONFIG_PRESETS_DESC"],
                         get = function() return TitanPals.Settings.TFormat; end,
                         set = function( _, value ) TitanPals.Settings.TFormat = value; end,
                         values = function()
                              local preList = {};
                              local v;
                              for _, v in pairs( TitanPalsStaticArray.presets ) do
                                   if ( v.value ~= TitanPals.Settings.TFormat ) then
                                        preList[v.value] = v.value;
                                   else
                                        preList[v.value] = ( "|cffffff9a%s|r" ):format( v.value );
                                   end
                              end
                              table.sort( preList, function( a, b )
                                   return string.lower( TitanPalsStaticArray.persets[a].name ) < string.lower( TitanPalsStaticArray.persets[b].name );
                              end );
                              return preList;
                         end
                    },
                    nulloption1 = {
                         type = "description", order = 2,
                         name = LB["TITAN_PALS_CONFIG_PRESETS_PREVIEW"],
                         cmdHidden = true,
                    },
               },
	  },
	  cat4 = {
               type = "group", order = 5,
               name = LB["TITAN_PALS_CONFIG_HEADER_ADD"],
               args = {
                    tooltipaddcustom = {
			 type = "input", order = 1, width = "full",
			 name = LB["TITAN_PALS_CONFIG_CUSTOM_TOOLTIP"],
			 desc = LB["TITAN_PALS_CONFIG_CUSTOM_TOOLTIP_DESC"],
			 get = function() return end,
			 set = function( _, value )
                              local tc = getn( TitanPalsStaticArray.presets );
			      tc = ( tc + 1 );
			      local tName = ( "Style #%s" ):format( tc );
			      local tValue = value;
			      TitanPanelPals_AddNewPreset( tName, tValue );
			 end,
		    },
                    nulloptionct = {
			 type = "description", order = 2,
			 name = LB["TITAN_PALS_CONFIG_CUSTOM_TOOLTIP_INST"],
			 cmdHidden = true;
                    },
               },
	  },
          cat5 = {
               type = "group", order = 6,
               name = LB["TITAN_PALS_CONFIG_HEADER_REMOVE"],
	       args = {
	            tooltippresetsremove = {
                         order = 1, type = "select", width = "full",
                         name = LB["TITAN_PALS_CONFIG_PRESETS"],
                         desc = LB["TITAN_PALS_CONFIG_REMOVE_PRESETS_DESC"],
                         get = function() return TitanPalsFormatToRemove; end,
                         set = function( _, value ) 
                              TitanPals.Temp["TitanPalsFormatToRemove"] = value;
                              TitanPalsFormatToRemove = value; 
			 end,
                         values = function()
                              local preList = {};
                              local v;
                              for _, v in pairs( TitanPalsStaticArray.presets ) do
                                   if ( v.value ~= TitanPals.Settings.TFormat ) then
                                        preList[v.value] = ( "%s %s" ):format( v.name, v.value );
                                   else
                                        preList[v.value] = ( "|cffffff9a%s %s|r" ):format( v.name, v.value );
                                   end
                              end
                              if ( TitanPalsFormatToRemove ~= "None" ) then
                                   preList["None"] = L["TITAN_NONE"];
                              else
                                   preList["None"] = ( "|cffffff9a%s|r" ):format( L["TITAN_NONE"] );
                              end
                              return preList;
                         end
                    },
		    tooltippresetremovebutton = {
                         type = "execute", order = 2,
			 name = LB["TITAN_PALS_CONFIG_REMOVE"],
			 desc = LB["TITAN_PALS_CONFIG_REMOVE_DESC"],
                         func = function()
                              if ( TitanPalsFormatToRemove == "None" ) then return; end
                              -- Need to add a check to see if the TFormat is about to be removed and if so change the TFormat to default
                              local k, v, z;
                              for k, v in pairs( TitanPalsStaticArray.presets ) do
                                   if ( TitanPals.Settings.TFormat == TitanPalsFormatToRemove and not z ) then 
                                        TitanPals.Settings.TFormat = v.value;
                                        z = true;
                                   end
                                   if ( v.value == TitanPalsFormatToRemove ) then
                                        table.remove( TitanPalsStaticArray.presets, k );
                                        TitanPalsFormatToRemove = "None";
                                        break;
                                   end
                              end
                         end,
		    },
               },
          },
	  cat6 = {
	       type = "group", order = 6,
               name = LB["TITAN_PALS_CONFIG_HEADER_STATUS"],
               args = {
                    selectt = {
                         type = "select", order = 1,
                         name = "Catagory Select",
                         desc = "test",
                         get = function() return TitanPals.Settings.dDefault;  end,
                         set = function( _, value ) TitanPals.Settings.dDefault = value; end,
                         values = function()
                              local db = { { name = "Away", value = "Away" }, { name = "Do Not Disturb", value = "DND", }, };
                              local preList = {};
                              local v;
                              for _, v in pairs( db ) do
                                   if ( v.value ~= TitanPals.Settings.dDefalut ) then
                                        preList[v.value] = v.value;
                                   else
                                        preList[v.value] = ( "|cffffff9a%s|r" ):format( v.value );
                                   end
                              end
                              return preList;
                         end,
                    },
                    scat1 = {
                         type = "group", order = 2, inline = true,
                         name = "Away",
                         hidden = function()
                              if ( TitanPals.Settings.dDefault == "Away" ) then
                                   return false;
                              else
                                   return true;
                              end
                         end,
                         args = {
                              statustypes = {
                                   type = "select", order = 1, width = "full",
                                   name = LB["TITAN_PALS_CONFIG_STATUS_TYPES"],
                                   desc = LB["TITAN_PALS_CONFIG_STATUS_TYPES_DESC"],
                                   get = function() return TitanPals.Settings.SFormat; end,
                                   set = function( _, value ) TitanPals.Settings.SFormat = value; end,
                                   values = function()
                                        local preList = {};
                                        local v;
                                        for _, v in pairs( TitanPals.sTypes ) do
                                             if ( v.value ~= TitanPals.Settings.SFormat ) then
                                                  preList[v.value] = v.value;
                                             else
                                                  preList[v.value] = ( "|cffffff9a%s|r" ):format( v.value );
                                             end
                                        end
                                        return preList;
                                   end
                              },
                              nulloptionct = {
                                   type = "description", order = 2,
                                   name = LB["TITAN_PALS_CONFIG_STATUS_TYPES_INST"],
                                   cmdHidden = true
                              },
                         },
                    },
                    scat2 = {
                         type = "group", order = 3, inline = true,
                         name = "Do Not Disturb",
                         hidden = function()
                              if ( TitanPals.Settings.dDefault == "DND" ) then
                                   return false;
                              else
                                   return true;
                              end
                         end,
                         args = {
                              statustypes = {
                                   type = "select", order = 1, width = "full",
                                   name = LB["TITAN_PALS_CONFIG_DND_TYPES"],
                                   desc = LB["TITAN_PALS_CONFIG_DND_TYPES_DESC"],
                                   get = function() return TitanPals.Settings.DFormat; end,
                                   set = function( _, value ) TitanPals.Settings.DFormat = value; end,
                                   values = function()
                                        local preList = {};
                                        local v;
                                        for _, v in pairs( TitanPals.dTypes ) do
                                             if ( v.value ~= TitanPals.Settings.DFormat ) then
                                                  preList[v.value] = v.value;
                                             else
                                                  preList[v.value] = ( "|cffffff9a%s|r" ):format( v.value );
                                             end
                                        end
                                        return preList;
                                   end
                              },
                              nulloptionct = {
                                   type = "description", order = 2,
                                   name = LB["TITAN_PALS_CONFIG_DND_TYPES_INST"],
                                   cmdHidden = true
                              },
                         },
                    },
               },
          },
          cat7 = {
	       type = "group", order = 7,
	       name =  LB["TITAN_PALS_CONFIG_HEADER_REALID"];
	       args = {
                    realidonoroff = {
			 name = LB["TITAN_PALS_CONFIG_REALID_ONOFF"],
			 desc = LB["TITAN_PALS_CONFIG_REALID_ONOFF_DESC"],
                         type = "toggle", order = 1, width = "full",
			 get = function() return TitanPals.Settings.ShowBNTooltip; end,
			 set = function( _, value ) 
                              TitanPals.Settings.ShowBNTooltip = value;
                              TitanPanelButton_UpdateButton( LB["TITAN_PALS_CORE_ID"] );
                         end,
		    },
		    tooltipacustomrealid = {
			 type = "select", order = 2, width = "full",
			 name = LB["TITAN_PALS_CONFIG_CUSTOM_TOOLTIP_REALID"],
			 desc = LB["TITAN_PALS_CONFIG_CUSTOM_TOOLTIP_REALID_DESC"],
			 get = function() return TitanPals.Settings.RealIDFormat end,
			 set = function( _, value ) TitanPals.Settings.RealIDFormat = value; end,
                         values = function()
                              local preList = {};
                              local v;
                              for _, v in pairs( TitanPals.realidformats ) do
                                   if ( v.value ~= TitanPals.Settings.RealIDFormat ) then
                                        preList[v.value] = v.value;
                                   else
                                        preList[v.value] = ( "|cffffff9a%s|r" ):format( v.value );
                                   end
                              end
                              return preList;
                         end
		    },
                    nulloption1 = {
                         type = "description", order = 3,
                         name = "\n",
                         cmdHidden = true,
                    },
                    confrealinondesc = {
                         type = "description", order = 4,
                         name = LB["TITAN_PALS_CONFIG_REALID_NOTE1_ON"],
                         cmdHidden = true,
                    },
                    nulloption2 = {
                         type = "description", order = 5,
                         name = "\n",
                         cmdHidden = true,
                    },
                    confrealidoffdesc = {
                         type = "description", order = 6,
                         name = LB["TITAN_PALS_CONFIG_REALID_NOTE1_OFF"],
                         cmdHidden = true,
                    },
               },
	  },
	  cat8 = {
               type = "group", order = 8,
               name = LB["TITAN_PALS_CONFIG_HEADER_REALID_ADD"],
               args = {
                    tooltipaddrealidcustom = {
			 type = "input", order = 1, width = "full",
			 name = LB["TITAN_PALS_CONFIG_CUSTOM_REALID_TOOLTIP"],
			 desc = LB["TITAN_PALS_CONFIG_CUSTOM_REALID_TOOLTIP_DESC"],
			 get = function() return end,
			 set = function( _, value )
                              local tc = getn( TitanPals.realidformats );
			      tc = ( tc + 1 );
			      local tName = ( "Format #%s" ):format( tc );
			      local tValue = value;
			      TitanPanelPals_AddNewRealIDPreset( tName, tValue );
			 end,
		    },
                    nulloptionct = {
			 type = "description", order = 2,
			 name = LB["TITAN_PALS_CONFIG_CUSTOM_REALID_TOOLTIP_INST"],
			 cmdHidden = true,
                    },
               },
	  },
          cat9 = {
               type = "group", order = 9,
               name = LB["TITAN_PALS_CONFIG_HEADER_REALID_REMOVE"],
	       args = {
	            tooltippresetsremove = {
                         type = "select", order = 1, width = "full",
                         name = LB["TITAN_PALS_CONFIG_REALID_PRESETS"],
                         desc = LB["TITAN_PALS_CONFIG_REALID_REMOVE_PRESETS_DESC"],
                         get = function() return TitanPalsFormatToRemove; end,
                         set = function( _, value ) 
                              TitanPals.Temp["TitanPalsFormatToRemove"] = value;
                              TitanPalsFormatToRemove = value;
			 end,
                         values = function()
                              local preList = {};
                              local v;
                              for _, v in pairs( TitanPals.realidformats ) do
                                   if ( v.value ~= TitanPals.Settings.RealIDFormat ) then
                                        preList[v.value] = ( "%s %s" ):format( v.name, v.value );
                                   else
                                        preList[v.value] = ( "|cffffff9a%s %s|r" ):format( v.name, v.value );
                                   end
                              end
                              if ( TitanPalsFormatToRemove ~= "None" ) then
                                   preList["None"] = L["TITAN_NONE"];
                              else
                                   preList["None"] = ( "|cffffff9a%s|r" ):format( L["TITAN_NONE"] );
                              end
                              return preList;
                         end,
                    },
		    tooltippresetremovebutton = {
                         type = "execute", order = 2,
			 name = LB["TITAN_PALS_CONFIG_REALID_REMOVE"],
			 desc = LB["TITAN_PALS_CONFIG_REALID_REMOVE_DESC"],
                         func = function()
                              if ( TitanPalsFormatToRemove == "None" ) then return; end
                              -- Need to add a check to see if the RealIDFormat is about to be removed and if so change the RealIDFormat to default
                              local k, v, z;
                              for k, v in pairs( TitanPals.realidformats ) do
                                   if ( TitanPals.Settings.RealIDFormat == TitanPalsFormatToRemove and not z ) then 
                                        TitanPals.Settings.RealIDFormat = v.value;
                                        z = true;
                                   end
                                   if ( v.value == TitanPalsFormatToRemove ) then
                                        table.remove( TitanPals.realidformats, k );
                                        TitanPalsFormatToRemove = "None";
                                        break;
                                   end
                              end
                         end,
		    },
               },
          },
          cat10 = {
               type = "group", order = 10,
               name = "Friend Edit",
               -- desc = "This will let you edit friend data",
               hidden = function()
                    eCheck = TitanPals_CoreUtils( "checkEdit", "_false" )
                    return not eCheck;
               end,
               args = {
                    toonName = {
                         type = "select", order = 1, -- width = "full",
                         name = "Friend",
                         desc = "Select toon to edit",
                         get = function() return TitanPals.toonEdit.toonName; end,
                         set = function( _, value ) TitanPals.toonEdit.toonName = value; end,
                         values = function()
                              local list, labels = {}, {};
                              local _, labels = TitanPals_CoreUtils( "checkEdit", "_true" );
                              tinsert( labels, { name = LB["TITAN_PALS_CONFIG_SELECT"], value = LB["TITAN_PALS_CONFIG_SELECT"], } )
                              for _, v in pairs( labels ) do
                                   list[v.value] = v.name;
                              end
                              return list;
			 end,
                    },
                    toonClass = {
                         type = "select", order = 2, -- width = "full",
                         name = "Class",
                         desc = "This will set this toons class",
                         get = function() return TitanPals.toonEdit.toonClass; end,
                         set = function( _, value ) TitanPals.toonEdit.toonClass = value; end,
                         values = function()
                              TitanPals.Temp = {};
                              local list, labels = {}, {};
                              local cClass, fClass, lClass, sClass;
                              for name, class in pairs( TitanPalsToEng ) do
                                   lClass = TitanPalsLocalClass[class];
                                   fClass = TitanPalsColorClass[class];
                                   cClass = fClass:format( lClass )
                                   tinsert( labels, { color = cClass, class = lClass, } )                                   
                              end
                              tinsert( labels, { color = "Unknown", class = "Unknown" } );
                              for _, v in pairs( labels ) do
                                   list[v.class] = v.color;
                              end
                              return list;
			 end,
                    },
                    toonLevel = {
                         type = "range", order = 3,
                         -- type = "select", order = 3,
                         -- type = "input", order = 3, -- width = "full",
                         name = "Level",
                         desc = "This will set this toons level",
                         get = function() return TitanPals.toonEdit.toonLevel; end,
                         set = function( _, value ) TitanPals.toonEdit.toonLevel = value; end,
			 min = LB["TITAN_PALS_CORE_MINLEVEL"],
			 max = LB["TITAN_PALS_CORE_MAXLEVEL"],
                         step = 1,
                         -- values = function()
                         --      local list = {}
                         --      local levels = {};
                         --      for i = 0, 85 do tinsert( levels, { name = i, value = i, } ); end
                         --      for _, v in pairs( levels ) do
                         --           list[v.name] = v.value;
                         --      end
                         --      return list;
			 -- end,
                    },
                    toonAlt = {
                         type = "select", order = 4, -- width = "full",
                         name = "Alt",
                         desc = "This will set this toons alt status",
                         get = function() return TitanPals.toonEdit.toonAlt; end,
                         set = function( _, value ) TitanPals.toonEdit.toonAlt = value; end,
                         values = function()
                              local list = {};
                              local labels = { { name = LB["TITAN_PALS_CONFIG_YES"], value = LB["TITAN_PALS_CONFIG_YES"], }, { name = LB["TITAN_PALS_CONFIG_NO"], value = LB["TITAN_PALS_CONFIG_NO"], }, };
                              for _, v in pairs( labels ) do
                                   list[v.name] = v.value;
                              end
                              return list;
			 end,
                    },
                    toonSpacer = {
                         type = "description", order = 5,
                         name = "\n",
                         cmdHidden = true,
                    },
                    toonSet = {
                         type = "execute", order = 6,
                         name = LB["TITAN_PALS_CONFIG_EDITCHAR"],
                         desc = "Edits character's info",
                         func = function()
                              if ( TitanPals.toonEdit.toonName ~= LB["TITAN_PALS_CONFIG_SELECT"] ) then
                                   if ( TitanPals.toonEdit.toonClass ~= LB["TITAN_PALS_CONFIG_UNKNOWN"] and TitanPals.toonEdit.toonLevel ~= 0 ) then
                                        local name = TitanPals.toonEdit.toonName;
                                        local class = TitanPals.toonEdit.toonClass;
                                        local level = TitanPals.toonEdit.toonLevel;
                                        TitanPals.Lists[palsDB][name].Class = class;
                                        TitanPals.Lists[palsDB][name].Level = level;
                                        if ( TitanPals.toonEdit.toonAlt == LB["TITAN_PALS_CONFIG_YES"] ) then
                                             TitanPals.Alts[palsDB][name] = class;
                                        end
                                        TitanPals.toonEdit = {
                                             toonName = "Select",
                                             toonClass = "Unknown",
                                             toonLevel = 0,
                                             toonAlt = "No",
                                        };
                                        collectgarbage( "collect" );
                                   else
                                        collectgarbage( "collect" );
                                   end
                              end
                         end,
                    },
               },
          },
          cat11 = {
               type = "group", order = 11,
               name = LB["TITAN_PALS_CORE_DEBUG_HEADER"],
               hidden = function() return not TitanPals.Debug.State; end,
               args = {
                    event = {
                         type = "toggle", order = 1,
                         name = LB["TITAN_PALS_CORE_DEBUG_EVENT"],
                         desc = LB["TITAN_PALS_CORE_DEBUG_EVENT_DESC"],
                         get = function() return TitanPals.Debug.Event; end,
                         set = function( _, value ) TitanPals.Debug.Event = value; end,
                    },
                    logevent = {
                         type = "toggle", order = 2,
                         name = LB["TITAN_PALS_CORE_DEBUG_LOGEVENT"],
                         desc = LB["TITAN_PALS_CORE_DEBUG_LOGEVENT_DESC"],
                         get = function() return TitanPals.Debug.LogEvent; end,
                         set = function( _, value ) TitanPals.Debug.LogEvent = value; end,
                    },
		    playfriends = {
                         type = "execute", order = 3,
			 name = LB["TITAN_PALS_CORE_DEBUG_PRINT_FRIENDS"],
			 desc = LB["TITAN_PALS_CORE_DEBUG_PRINT_FRIENDS_DESC"],
                         func = function()
                              TitanPals_CoreUtils( "playFriends" );
                         end,
		    },
		    playalts = {
                         type = "execute", order = 4,
			 name = LB["TITAN_PALS_CORE_DEBUG_PRINT_ALTS"],
			 desc = LB["TITAN_PALS_CORE_DEBUG_PRINT_ALTS_DESC"],
                         func = function()
                              TitanPals_CoreUtils( "playAlts" );
                         end,
		    },

		    remauth = {
                         type = "execute", order = 5,
			 name = LB["TITAN_PALS_CORE_DEBUG_REMAUTH"],
			 desc = function()
                              local author = ( "%s@%s" ):format( TitanPals.Profile.Player, TitanPals.Profile.Realm )
                              return LB["TITAN_PALS_CORE_DEBUG_REMAUTH_DESC"]:format( author )
                         end,
                         func = function()
                              TitanPals_CoreUtils( "RemAuth" );
                              TitanPals.Debug.State = false;
                              TitanPals.Debug.Event = false;
                              TitanPals.Debug.LogEvent = false;
                              --InterfaceOptionsFrame_OpenToCategory( LB["TITAN_PALS_ADDON_LABEL"] );
                         end,
		    },

               },
          },
     },
}

-- **************************************************************************
-- NAME : TitanPanelPals_AddNewPreset()
-- DESC : This will add a custom tooltip preset
-- **************************************************************************
function TitanPanelPals_AddNewPreset( presetname, presetvalue )
     local found
     for _,i in pairs( TitanPals.presets ) do
          if i.name == presetname or i.value == presetvalue then
               found = true; break;
          end
     end	
     if ( not found ) then table.insert( TitanPals.presets, { name = presetname, value = presetvalue } ); end
end

-- **************************************************************************
-- NAME : TitanPanelPals_AddNewRealIDPreset()
-- DESC : This will add a custom tooltip preset for RealID Display
-- **************************************************************************
function TitanPanelPals_AddNewRealIDPreset( presetname, presetvalue )
     local found
     for _, i in pairs( TitanPals.realidformats ) do
          if ( i.name == presetname or i.value == presetvalue ) then
               found = true;
               break;
          end
     end	
     if ( not found ) then table.insert( TitanPals.realidformats, { name = presetname, value = presetvalue } ); end
end

-- **************************************************************************
-- NAME : TitanPanelPalsButton_OnLoad()
-- DESC : Registers the plugin upon it loading
-- **************************************************************************
function TitanPanelPalsButton_OnLoad( self )
     self.registry = {
          id = LB["TITAN_PALS_CORE_ID"],
          builtIn = false,
          version = TITAN_VERSION,
          menuText = LB["TITAN_PALS_MENU_TEXT"],
          buttonTextFunction = "TitanPanelPalsButton_GetButtonText";
	  tooltipTitle = LB["TITAN_PALS_TOOLTIP"],
          tooltipCustomFunction = TitanPanelPals_SetTooltip;
          icon = LB["TITAN_PALS_CORE_ARTWORK"],
          iconWidth = 16,
          category = "Information",
          savedVariables = {
	       ShowIcon = 1,
	       ShowLabelText = 1,
          }
     };
     self:RegisterEvent( "ADDON_LOADED" );
     self:RegisterEvent( "PLAYER_ENTERING_WORLD" );
     self:RegisterEvent( "BN_FRIEND_ACCOUNT_ONLINE" );
     self:RegisterEvent( "BN_FRIEND_ACCOUNT_OFFLINE" );
     self:RegisterEvent( "FRIENDLIST_UPDATE" );
     self:RegisterEvent( "IGNORELIST_UPDATE" );

     -- Titan Pals Config
     AceConfig:RegisterOptionsTable( PlugInName, PalsOpt );
     AceConfigDialog:AddToBlizOptions( PlugInName, LB["TITAN_PALS_ADDON_LABEL"] );
     if ( not TitanPalsInit or TitanPalsInit == nil ) then
          TitanPals_Init( true );
     else
          TitanPals_Init( false );
     end
end

-- **************************************************************************
-- NAME : SLASH_TITANPALS*
-- DESC : Slash command setup
-- **************************************************************************
SLASH_TITANPALS1 = "/titanpals";
SLASH_TITANPALS2 = "/tp";
SlashCmdList["TITANPALS"] = function( msg )
     TitanPals_CommandHandler( "titanpals", msg );
end

-- **************************************************************************
-- NAME : TitanPals_ShowHelp( cmd )
-- DESC : Slash command help display
-- **************************************************************************
function TitanPals_ShowHelp( cmd )
     TitanPals_DebugUtils( "info", "help", LB["TITAN_PALS_HELP_LINE1"] );
     TitanPals_DebugUtils( "info", "help", LB["TITAN_PALS_HELP_LINE2"]:format( cmd ) );
     TitanPals_DebugUtils( "info", "help", LB["TITAN_PALS_HELP_LINE3"]:format( cmd ) );
     TitanPals_DebugUtils( "info", "help", LB["TITAN_PALS_HELP_LINE4"]:format( cmd ) );
     TitanPals_DebugUtils( "info", "help", LB["TITAN_PALS_HELP_LINE5"]:format( cmd ) );
     local auth = TitanPals_CoreUtils( "Author", ( "%s@%s" ):format( TitanPals.Profile.Player, TitanPals.Profile.Realm ) )
     if ( auth ) then
          TitanPals_DebugUtils( "info", "help", LB["TITAN_PALS_HELP_ADDAUTH"]:format( cmd ) );
          TitanPals_DebugUtils( "info", "help", LB["TITAN_PALS_HELP_REMAUTH"]:format( cmd ) );
     end
end

-- **************************************************************************
-- NAME : TitanPals_CommandHandler()
-- DESC : Slash command handler
-- **************************************************************************
function TitanPals_CommandHandler( slash, command )
     local _, _, cmd1, cmd2 = strfind( command, "^([^ ]+) (.+)$" );
     if ( not cmd1 ) then cmd1 = command; end
     --if ( not cmd1 ) then cmd1 = ""; end
     --if ( not cmd2 ) then cmd2 = ""; end
     if ( ( cmd1 == "" ) or ( cmd1 == "help" ) ) then
          TitanPals_ShowHelp( slash );
     elseif ( cmd1 == "options" or cmd1 == "config" ) then
          InterfaceOptionsFrame_OpenToCategory( LB["TITAN_PALS_ADDON_LABEL"] );
     elseif ( cmd1 == "reset" ) then
          TitanPals_ResetUtils( true, cmd1 );
     elseif ( cmd1 == "checkDB" ) then
          TitanPals_ResetUtils( false, cmd1 );
     elseif ( cmd1 == "AddAuth" or cmd1 == "addauth" ) then
          TitanPals_CoreUtils( cmd1 );
     elseif ( cmd1 == "RemAuth" or cmd1 == "remauth" ) then
          TitanPals_CoreUtils( cmd1 );
     elseif ( cmd1 == "Test" or cmd1 == "test" ) then
     -- end
          if ( slash ) then print( ( "slash = %s" ):format( slash ) ) end;
          if ( cmd1 ) then print( ( "cmd1 = %s" ):format( cmd1 ) ); end
          if ( command ) then print( ( "command = %s" ):format( command ) ) end
          if ( cmd2 ) then print( ( "cmd2 = %s" ):format( cmd2 ) ) end;
          TitanPals.Temp = {};
          n, v = TitanPals_CoreUtils( "checkEdit", true );
     else
          TitanPals_DebugUtils( "error", "CommandHandler", LB["TITAN_PALS_HELP_ERROR"]:format( cmd1 ) )
     end
end